/*
 * DimensionImageObserver.java
 * Part of the B'am library - a collection of general-purpose utility classes.
 * 
 * Copyright (C)2006 Stephen Battey
 * Contact: resource@lucidviews.net
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

package net.lucidviews.util.image;

import java.awt.image.ImageObserver;
import java.awt.Image;


/**
 * An <code>ImageObserver</code> that notifies a listener when the dimension
 * information for a given image has loaded.
 * <br />
 * <br />
 * @author Stephen Battey
 * @version $Revision: 1.2 $
 * @since 1.0
 */
public class DimensionImageObserver extends BaseImageObserver
{
	
	/*
	 * Instance variables
	 */
	
	/** The listener that wants to be told of the image's dimension. */
	protected Listener _listener;

	
	/*
	 * Constructors
	 */
	
	/**
	 * Create an observer that informs the listener of the image's dimension.
	 * <br />
	 * <br />
	 * @param listener the listener waiting for the dimension height information
	 */
	public DimensionImageObserver( Listener listener )
	{
		super();
		_listener = listener;
	}
	
	/**
	 * Create an observer that informs the listener of the specified image's
	 * dimension.
	 * <br />
	 * The extra <code>image</code> parameter restricts this observer such that
	 * it only informs the listener when the specified image has loaded and not
	 * any other image it has been assigned to observe.
	 * <br />
	 * <br />
	 * @param listener the listener waiting for the dimension height information
	 * @param image the image that is to be monitored
	 */
	public DimensionImageObserver( Listener listener, Image image )
	{
		super( image );
		_listener = listener;
	}
	
	
	/*
	 * Implementation of abstract methods
	 */
	
	public boolean observedImageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	{
		boolean widthLoaded = (infoflags & ImageObserver.WIDTH) == ImageObserver.WIDTH;
		boolean heightLoaded = (infoflags & ImageObserver.HEIGHT) == ImageObserver.HEIGHT;
		
		if ( (widthLoaded) && (heightLoaded) )
		{
			_listener.setImageDimension( width, height );
			
			// all done
			return false;
		}
		else
		{
			// still waiting for dimension information to load
			return true;
		}
	}
	
	
	/*
	 * Helper classes
	 */
	
	/**
	 * This interface should be implemented by classes that want to be told the
	 * image's width and height when that information has become available.
	 */
	public static interface Listener
	{
		
		/**
		 * This method will be called by a <code>DimensionImageObserver</code>
		 * when the dimension information has loaded.
		 * <br />
		 * <br />
		 * @param width the width of the image in pixels
		 * @param height the height of the image in pixels
		 */
		public void setImageDimension( int width, int height );
		
	}
	
}
